import { Style } from '../style';
import { IStyles } from './IStyles';
/**
 * This class represents the automatic styles of a document.
 *
 * An automatic style contains formatting properties that are considered to be dedicated to a single element.
 *
 * @example
 * const automaticStyles = new AutomaticStyles();
 * automaticStyles.add(new ParagraphStyle());
 * automaticStyles.getName(new ParagraphStyle());
 * automaticStyles.getAll();
 *
 * @since 0.9.0
 * @private
 */
export declare class AutomaticStyles implements IStyles {
    private styles;
    private listStyleCounter;
    private paragraphStyleCounter;
    /**
     * Creates a `AutomaticStyles` instance that represents the automatic styles of a document.
     *
     * @example
     * const automaticStyles = new AutomaticStyles();
     *
     * @since 0.9.0
     */
    constructor();
    /**
     * The `add()` method adds the given style to the list of automatic styles and a unique name will be assigned to it.
     *
     * If an equal style already was added the current style will be ignored.
     *
     * @example
     * const automaticStyles = new AutomaticStyles();
     * const style1 = new ParagraphStyle();
     * const style2 = new ParagraphStyle().setFontSize(23);
     * automaticStyles.add(style1); // 'P1'
     * automaticStyles.add(style2); // 'P2'
     * automaticStyles.add(style1); // ignore
     *
     * @param {Style} style The style which should be added to the list of automatic styles
     * @since 0.9.0
     */
    add(style: Style): void;
    /**
     * The `getName()` method returns the unique name of the given style.
     *
     * @example
     * const automaticStyles = new AutomaticStyles();
     * const style1 = new ParagraphStyle();
     * const style2 = new ParagraphStyle().setFontSize(23);
     * automaticStyles.add(style1);
     * automaticStyles.getName(style1); // 'P1'
     * automaticStyles.getName(style2); // error
     *
     * @param {Style} style The style for which the unique name is being requested
     * @returns {string | never} The unique name of the style
     * @throws {Error} If the style has not been added to the list of automatic styles
     * @since 0.9.0
     */
    getName(style: Style): string | never;
    /** @inheritdoc */
    getAll(): Style[];
    /**
     * Creates the unique name for the given style.
     * The name follows the pattern <type><count> (e.g. P2 for the second paragraph style).
     *
     * @param {Style} style The style for which a unique name is being requested
     * @returns {string} The unique name representing the given style
     */
    private createUniqueName;
    /**
     * The `getHash()` method returns a representative hash for a given style.
     *
     * @param {Style} style The style for which a representative hash is being requested
     * @returns {string} The hash representing the given style
     */
    private getHash;
    /**
     * Updates the hash with the list level style.
     *
     * @param {Hash} hash The hash to update
     * @param {BulletListLevelStyle} listLevelStyle The list level style to evaluate
     */
    private updateHashWithListLevelStyle;
    /**
     * Updates the hash with the paragraph properties.
     *
     * @param {Hash} hash The hash to update
     * @param {IParagraphProperties} paragraphProperties The paragraph properties to evaluate
     */
    private updateHashWithParagraphProperties;
    /**
     * Updates the hash with the text properties.
     *
     * @param {Hash} hash The hash to update
     * @param {ITextProperties} textProperties The text properties to evaluate
     */
    private updateHashWithTextProperties;
}
